# -*- coding: utf-8 -*-
import json
from django.conf import settings
from django.utils.translation import ugettext as _
from django.contrib import admin
from django.conf.urls import patterns, url
from django.http import HttpResponse, Http404
from celery.result import AsyncResult
from celery import states
from jmb.core.admin.options import ExtendibleModelAdmin
from jmb.async.models import Task

if settings.DEBUG:
    from jmb.async.tasks import AspettaTask


def JsonResponse(response):
    return HttpResponse(json.dumps(response), content_type='application/json')

class TaskAdmin(ExtendibleModelAdmin):  
    fieldsets = (
         (_('logs admin'), {
		'fields': ('creator', 'date_create', 'last_modifier', 'date_last_modify',),
		'classes': ('collapse',)
	}),
	(None, {
		'fields': (
        'task_id', 'app_label', 'title', 'status', 'note'
	)}),

    )
    list_display = (
       'task_id', 'app_label', 'get_progress_bar', 'title', 'note', 'get_edit_icon', 'get_delete_icon',
    )
    
    def dummy_task(modeladmin, request, queryset):
        result = AspettaTask.delay_or_fail()
    dummy_task.short_description = "Create test task"
    
    def get_actions(self, request):
        actions = super(TaskAdmin, self).get_actions(request)
        if not settings.DEBUG:
            del actions['dummy_task']
        return actions
    
    actions = [dummy_task]

    def get_urls(self):
        urls = super(TaskAdmin, self).get_urls()
        my_urls = patterns('',
            url(r'^(?P<task_id>[\w\d\-\.]+)/status/$', self.task_status,
                name='task_status'),
        )
        return my_urls + urls
        
    def task_status(self, request, task_id):
        """Returns task status and result in JSON format."""
        result = AsyncResult(task_id)
        state, retval = result.state, result.result
        response_data = dict(id=task_id, status=state, result=retval)
        if state in states.EXCEPTION_STATES:
            traceback = result.traceback
            response_data.update({'result': safe_repr(retval),
                                  'exc': get_full_cls_name(retval.__class__),
                                  'traceback': traceback})
        return JsonResponse({'task': response_data})
 
admin.site.register(Task, TaskAdmin)
